%Calcula la posicion de los hitos kilometricos en la carretera teniendo
%en cuenta que deben estar separados la misma distancia
%
%  T = kilometra_carretera(carretera) devuelve un vector con los distintos
%    valores de t donde coincide cada hito kilomtrico separados una distancia
%    por defecto de 1 Kilmetro.
%
%  T = kilometra_carretera(carretera,distancia) especifica un valor de distancia
%    diferente a la distancia por defecto.
%
%  [T,puntos,Iter] = kilometra_carretera(carretera) devuelve adems un vector puntos
%    con los puntos evaluados en cada t y un vector con el nmero de iteraciones
%    que fueron necesarias para calcular cada valor.
%
function [T,puntos,Iter,PASO_t] = kilometra_carretera(c,distancia)
    if nargin < 2
        distancia = 1; % por defecto, separamos cada hito una unidad
    end
    
    t0 = 1; %empezamos a hacer el recorrido desde el principio
    nhitos = 1;
    T(1) = t0; %t's para cada punto
    Iter(1) = 0; %numero de iteraciones del mtodo para cada punto
    tfinal = c.N;
    TOL = 0.000001*c.escala; 
    MAX_NEWTON_ITER = 40; %nme mximo de iteraciones para Newton-Rhapson
    h = 0.01; %para derivar

    PASO_t = 1;
    
    fprintf('\nKilometrando la carretera "%s" a escala x,y = %d Km. [Usando paso t = %d]\n',c.nombrefichero,c.escala,PASO_t);
    fprintf('Calculando: .');

    sti_hitos = fix(longitud_carretera(c) / (distancia - TOL)); %numero de hitos estimados que se asignarn

    %Buscamos hitos hasta que lleguemos al final de la carretera 
    %o la carretera no sea suficientemente larga
    niter_hitos = 0; %limite de iteraciones del metodo
    while t0 < tfinal && (niter_hitos < sti_hitos || longitud_carretera(c,t0) >= (distancia-TOL)) && niter_hitos < 1000
	%avanzamos hasta acercarnos suficiente al t solucin
	t = fix(t0) + PASO_t; %punto inicial
	while longitud_carretera(c,t0,t) < distancia && t < tfinal
	    t = t + PASO_t;
        end
	t = min(t,tfinal); %por si rebasamos el lmite

	% aplicamos Newton-Rhapson para refinar t tal que
	% f(t) = longitud_carretera(c,t0,t) - distancia = 0
	% ti+1 = ti - f(ti)/f'(ti)
	niter = 0;
	f = longitud_carretera(c,t0,t) - distancia;
	while abs(f) > TOL && niter < MAX_NEWTON_ITER
	    derivada_f = (longitud_carretera(c,t0,t+h) - longitud_carretera(c,t0,t-h))/(2*h);    
	    t = t - f/derivada_f;
	    f = longitud_carretera(c,t0,t) - distancia; %evaluamos nueva f
	    niter = niter + 1;
	end
	assert(t>0,'t tiene que ser positiva');
	assert(t>t0,'t tiene que ser mayor que t0 en cada paso');
	if t <= tfinal + TOL
	   %assert(niter < MAX_NEWTON_ITER,'mximo de interaciones rebasado. Posiblemente el mtodo no haya convergido');
	   nhitos = nhitos + 1;
	   T(nhitos) = t;
	   Iter(nhitos) = niter;
	   fprintf('.');
	end
	t0 = t;
    	niter_hitos = niter_hitos + 1;
    end

	fprintf('\n');
    puntos = punto_carretera(c,T); %evaluamos x e y para cada t

return;
