% crea la estructura carretera con todos lo necesario para trabajar 
% con estas
%
% c = LEE_CARRETERA(nombrefichero) lee a partir del fichero especificado los puntos 
%  de la carretera
%
% c = LEE_CARRETERA(X,Y) genera la estructura carretera a partir de los vectores X e Y
%
%
% La funcin devuelve la estructura CARRETERA, que contiene los siguientes campos:
%
%   x,y: Puntos X e Y iniciales de la carretera, para cada t.
%
%   N:   Nmero de puntos. => t comprendido entre 1 y N.
%
%   escala:   Nmero de Kilmetros que representa cada unidad.Tiene que ser positivo
%
%   nintervalos: Nmero de intervalos en los que est dividida la carretera = N - 1.
%
%   nombrefichero: Nombre del fichero donde fueron leidos los puntos de la carretera.
%
%   splineX: A su vez, es una estructura que almacena el spline cbico de X = f(t). Sus campos ms importantes son:
%       order: Nmero de coeficientes por polinomio interpolador.
%       breaks: valores de t donde se define cada x o y segn el caso.
%       pieces: Nmero de intervalos o polinmios = nintervalos.
%       coefs: contiene los coeficientes a,b,c,d para los polinomios interpoladores de cada intervalo.
%
%   splineY: contiene el spline cbico para Y = g(t).
%
function carretera = lee_carretera(arg1,arg2,arg3)

	%tratamos parametros de entrada
	if nargin < 3 && isa(arg1,'char')
	   % Cargamos los datos del trazado desde el fichero indicado
	   nombrefichero = arg1;
	   puntos_leidos = load(nombrefichero, '-ascii'); 
	   X = puntos_leidos(:,1);
	   Y = puntos_leidos(:,2);
	   if nargin < 2
		  escala = 1; %valor por defecto
	   else
	      escala = arg2;
   	   end
	elseif nargin > 1
		X = vector_columna(arg1);
		Y = vector_columna(arg2);
		nombrefichero = '[introducido a mano]';
	    if length(X) ~= length(Y)
			error('el tamao de los vectores tiene que ser el mismo');
	    end
		if nargin < 3
			escala = 1; %valor por defecto
		else
			escala = arg3;
	    end
	else
		error('numero de argumentos erroneo. consulte la ayuda');
	end

	assert(escala > 0,'la escala tiene que ser mayor que 0');

	%creamos estructura
	carretera.nombrefichero = nombrefichero;
	carretera.escala = escala; %escala de la carretera
	carretera.x = X; %valores de x para los puntos leidos
	carretera.y = Y; %valores de y para los puntos leidos
	carretera.N = size(carretera.x,1); %numero de puntos
	carretera.nintervalos = carretera.N -1;  %numero de intervalos (N-1)

	T = 1:carretera.N; 

	% Obtenemos los splines que interpolan al trazado en... 
	carretera.splineX = spline(T,carretera.x); %X(t)
	carretera.splineY = spline(T,carretera.y); %Y(t)

return;
